"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const path = require("path");
const ts = require("typescript");
const vscode_languageserver_1 = require("vscode-languageserver");
let strict = false;
/**
 * Toggles "strict" flag, affects how we are parsing/generating URLs.
 * In strict mode we using "file://PATH", otherwise on Windows we are using "file:///PATH"
 */
function setStrict(value) {
    strict = value;
}
exports.setStrict = setStrict;
function formEmptyRange() {
    return vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(0, 0), vscode_languageserver_1.Position.create(0, 0));
}
exports.formEmptyRange = formEmptyRange;
function formEmptyPosition() {
    return vscode_languageserver_1.Position.create(0, 0);
}
exports.formEmptyPosition = formEmptyPosition;
function formEmptyKind() {
    return vscode_languageserver_1.SymbolKind.Namespace;
}
exports.formEmptyKind = formEmptyKind;
/**
 * Makes documentation string from symbol display part array returned by TS
 */
function docstring(parts) {
    return ts.displayPartsToString(parts);
}
exports.docstring = docstring;
/**
 * Normalizes path to match POSIX standard (slashes)
 */
function toUnixPath(filePath) {
    return filePath.replace(/\\/g, '/');
}
exports.toUnixPath = toUnixPath;
function convertStringtoSymbolKind(kind) {
    switch (kind) {
        case 'module': return vscode_languageserver_1.SymbolKind.Module;
        case 'class': return vscode_languageserver_1.SymbolKind.Class;
        case 'local class': return vscode_languageserver_1.SymbolKind.Class;
        case 'interface': return vscode_languageserver_1.SymbolKind.Interface;
        case 'enum': return vscode_languageserver_1.SymbolKind.Enum;
        case 'enum member': return vscode_languageserver_1.SymbolKind.Constant;
        case 'var': return vscode_languageserver_1.SymbolKind.Variable;
        case 'local var': return vscode_languageserver_1.SymbolKind.Variable;
        case 'function': return vscode_languageserver_1.SymbolKind.Function;
        case 'local function': return vscode_languageserver_1.SymbolKind.Function;
        case 'method': return vscode_languageserver_1.SymbolKind.Method;
        case 'getter': return vscode_languageserver_1.SymbolKind.Method;
        case 'setter': return vscode_languageserver_1.SymbolKind.Method;
        case 'property': return vscode_languageserver_1.SymbolKind.Property;
        case 'constructor': return vscode_languageserver_1.SymbolKind.Constructor;
        case 'parameter': return vscode_languageserver_1.SymbolKind.Variable;
        case 'type parameter': return vscode_languageserver_1.SymbolKind.Variable;
        case 'alias': return vscode_languageserver_1.SymbolKind.Variable;
        case 'let': return vscode_languageserver_1.SymbolKind.Variable;
        case 'const': return vscode_languageserver_1.SymbolKind.Constant;
        case 'JSX attribute': return vscode_languageserver_1.SymbolKind.Property;
        // case 'script'
        // case 'keyword'
        // case 'type'
        // case 'call'
        // case 'index'
        // case 'construct'
        // case 'primitive type'
        // case 'label'
        // case 'directory'
        // case 'external module name'
        // case 'external module name'
        default: return vscode_languageserver_1.SymbolKind.Variable;
    }
}
exports.convertStringtoSymbolKind = convertStringtoSymbolKind;
function path2uri(root, file) {
    let ret = 'file://';
    if (!strict && process.platform === 'win32') {
        ret += '/';
    }
    let p;
    if (root) {
        p = resolve(root, file);
    }
    else {
        p = file;
    }
    p = toUnixPath(p).split('/').map(encodeURIComponent).join('/');
    return ret + p;
}
exports.path2uri = path2uri;
function uri2path(uri) {
    if (uri.startsWith('file://')) {
        uri = uri.substring('file://'.length);
        if (process.platform === 'win32') {
            if (!strict) {
                uri = uri.substring(1);
            }
        }
        uri = uri.split('/').map(decodeURIComponent).join('/');
    }
    return uri;
}
exports.uri2path = uri2path;
function uri2reluri(uri, root) {
    return path2uri('', uri2relpath(uri, root));
}
exports.uri2reluri = uri2reluri;
function uri2relpath(uri, root) {
    uri = uri2path(uri);
    root = toUnixPath(root);
    if (uri.startsWith(root)) {
        uri = uri.substring(root.length);
    }
    while (uri.startsWith('/')) {
        uri = uri.substring(1);
    }
    return uri;
}
exports.uri2relpath = uri2relpath;
function isLocalUri(uri) {
    return uri.startsWith('file://');
}
exports.isLocalUri = isLocalUri;
function resolve(root, file) {
    if (!strict || os.platform() !== 'win32') {
        return path.resolve(root, file);
    }
    else {
        return path.posix.resolve(root, file);
    }
}
exports.resolve = resolve;
let jstsPattern = /\.[tj]sx?$/;
function isJSTSFile(filename) {
    return jstsPattern.test(filename);
}
exports.isJSTSFile = isJSTSFile;
let jstsConfigPattern = /(^|\/)[tj]sconfig\.json$/;
function isConfigFile(filename) {
    return jstsConfigPattern.test(filename);
}
exports.isConfigFile = isConfigFile;
let packageJsonPattern = /(^|\/)package\.json$/;
function isPackageJsonFile(filename) {
    return packageJsonPattern.test(filename);
}
exports.isPackageJsonFile = isPackageJsonFile;
const globalTSPatterns = [
    /(^|\/)globals?\.d\.ts$/,
    /node_modules\/\@types\/node\/.*/,
    /(^|\/)typings\/.*/,
    /(^|\/)tsd\.d\.ts($|\/)/
];
// isGlobalTSFile returns whether or not the filename contains global
// variables based on a best practices heuristic
// (https://basarat.gitbooks.io/typescript/content/docs/project/modules.html). In
// reality, a file has global scope if it does not begin with an
// import statement, but to check this, we'd have to read each
// TypeScript file.
function isGlobalTSFile(filename) {
    for (const globalTSPattern of globalTSPatterns) {
        if (globalTSPattern.test(filename)) {
            return true;
        }
    }
    return false;
}
exports.isGlobalTSFile = isGlobalTSFile;
function isDependencyFile(filename) {
    return filename.startsWith('node_modules/') || filename.indexOf('/node_modules/') !== -1;
}
exports.isDependencyFile = isDependencyFile;
function isDeclarationFile(filename) {
    return filename.endsWith('.d.ts');
}
exports.isDeclarationFile = isDeclarationFile;
/**
 * Converts filename to POSIX-style absolute one if filename does not denote absolute path already
 */
function absolutize(filename) {
    filename = toUnixPath(filename);
    // If POSIX path does not treats filename as absolute, let's try system-specific one
    if (!path.posix.isAbsolute(filename) && !path.isAbsolute(filename)) {
        filename = '/' + filename;
    }
    return filename;
}
exports.absolutize = absolutize;
/**
 * Absolutizes directory name and cuts trailing slashes
 */
function normalizeDir(dir) {
    dir = absolutize(dir);
    if (dir !== '/') {
        dir = dir.replace(/[\/]+$/, '');
    }
    return dir;
}
exports.normalizeDir = normalizeDir;
/**
 * defInfoToSymbolDescriptor converts from an instance of
 * ts.DefinitionInfo to an instance of rt.SymbolDescriptor
 */
function defInfoToSymbolDescriptor(d) {
    return {
        kind: d.kind || '',
        name: stripQuotes(d.name) || '',
        containerKind: d.containerKind || '',
        containerName: (d.containerName ? stripFileInfo(lastDotCmp(stripQuotes(d.containerName))) : '')
    };
}
exports.defInfoToSymbolDescriptor = defInfoToSymbolDescriptor;
function symbolDescriptorMatch(query, sym) {
    for (const key of Object.keys(query)) {
        if (query[key] === undefined) {
            continue;
        }
        if (key === 'package') {
            if (!sym.package || !packageDescriptorMatch(query.package, sym.package)) {
                return false;
            }
            continue;
        }
        if (query[key] !== sym[key]) {
            return false;
        }
    }
    return true;
}
exports.symbolDescriptorMatch = symbolDescriptorMatch;
function packageDescriptorMatch(query, sym) {
    for (const key of Object.keys(query)) {
        if (query[key] === undefined) {
            continue;
        }
        if (query[key] !== sym[key]) {
            return false;
        }
    }
    return true;
}
function stripQuotes(s) {
    if ((s.startsWith('"') && s.endsWith('"')) || (s.startsWith("'") && s.endsWith("'"))) {
        return s.substring(1, s.length - 1);
    }
    return s;
}
function lastDotCmp(s) {
    const cmps = s.split('.');
    return cmps[cmps.length - 1];
}
/**
 * Strips file part (if any) from container name (last component of container path)
 * For example TS may return the following name: /node_modules/vscode-jsonrpc/lib/cancellation.
 * We consider that if name contains path separtor then container name is empty
 * @param containerName
 */
function stripFileInfo(containerName) {
    return toUnixPath(containerName).indexOf('/') < 0 ? containerName : '';
}
//# sourceMappingURL=util.js.map